@@ -149,19 +149,20 @@ static uint32_t parse_ascii_hex32(const char *s)
 
 /**
  * decode_programmer_archive() - Attempt to decode a programmer CPIO archive
- * @images: List of Sahara images, with @images[0] populated
+ * @blob: Loaded image to be decoded as archive
+ * @images: List of Sahara images to populate
  *
- * The single blob provided in @images[0] might be a CPIO archive containing
- * Sahara images, in files with names in the format "<id>:<filename>". Load
- * each such Sahara image into the relevant spot in the @images array.
+ * The blob might be a CPIO archive containing Sahara images, in files with
+ * names in the format "<id>:<filename>". Load each such Sahara image into the
+ * relevant spot in the @images array.
  *
- * The original blob (in @images[0]) is freed once it has been consumed.
+ * The blob is always consumed (freed) on both success and error paths.
+ * On error, any partially-populated @images entries are also freed.
  *
  * Returns: 0 if no archive was found, 1 if archive was decoded, -1 on error
  */
-static int decode_programmer_archive(struct sahara_image *images)
+static int decode_programmer_archive(struct sahara_image *blob, struct sahara_image *images)
 {
-	struct sahara_image *blob = &images[0];
 	struct cpio_newc_header *hdr;
 	size_t filesize;
 	size_t namesize;
@@ -178,13 +179,13 @@ static int decode_programmer_archive(struct sahara_image *images)
 	for (;;) {
 		if (ptr + sizeof(*hdr) > end) {
 			ux_err("programmer archive is truncated\n");
-			return -1;
+			goto err;
 		}
 		hdr = ptr;
 
 		if (memcmp(hdr->c_magic, "070701", 6)) {
 			ux_err("expected cpio header in programmer archive\n");
-			return -1;
+			goto err;
 		}
 
 		filesize = parse_ascii_hex32(hdr->c_filesize);
@@ -193,12 +194,12 @@ static int decode_programmer_archive(struct sahara_image *images)
 		ptr += sizeof(*hdr);
 		if (ptr + namesize > end || ptr + filesize + namesize > end) {
 			ux_err("programmer archive is truncated\n");
-			return -1;
+			goto err;
 		}
 
 		if (namesize > sizeof(name)) {
 			ux_err("unexpected filename length in progammer archive\n");
-			return -1;
+			goto err;
 		}
 		memcpy(name, ptr, namesize);
 
@@ -209,7 +210,7 @@ static int decode_programmer_archive(struct sahara_image *images)
 		id = strtoul(tok, NULL, 0);
 		if (id == 0 || id >= MAPPING_SZ) {
 			ux_err("invalid image id \"%s\" in programmer archive\n", tok);
-			return -1;
+			goto err;
 		}
 
 		ptr += namesize;
@@ -231,10 +232,18 @@ static int decode_programmer_archive(struct sahara_image *images)
 	blob->len = 0;
 
 	return 1;
+
+err:
+	sahara_images_free(images, MAPPING_SZ);
+	free(blob->ptr);
+	blob->ptr = NULL;
+	blob->len = 0;
+	return -1;
 }
 
 /**
  * decode_sahara_config() - Attempt to decode a Sahara config XML document
+ * @blob: Loaded image to be decoded as Sahara config
  * @images: List of Sahara images, with @images[0] populated
  *
  * The single blob provided in @images[0] might be a XML blob containing
@@ -246,9 +255,8 @@ static int decode_programmer_archive(struct sahara_image *images)
  *
  * Returns: 0 if no archive was found, 1 if archive was decoded, -1 on error
  */
-static int decode_sahara_config(struct sahara_image *images)
+static int decode_sahara_config(struct sahara_image *blob, struct sahara_image *images)
 {
-	struct sahara_image *blob = &images[0];
 	char image_path_full[PATH_MAX];
 	const char *image_path;
 	unsigned int image_id;
@@ -335,6 +343,10 @@ static int decode_sahara_config(struct sahara_image *images)
 	return 1;
 
 err_free_doc:
+	sahara_images_free(images, MAPPING_SZ);
+	free(blob->ptr);
+	blob->ptr = NULL;
+	blob->len = 0;
 	xmlFreeDoc(doc);
 	free(blob_name_buf);
 	return -1;
@@ -344,15 +356,15 @@ err_free_doc:
  * decode_programmer() - decodes the programmer specifier
  * @s: programmer specifier, from the user
  * @images: array of images to populate
- * @single: legacy single image specifier, for which image id should be ignored
  *
  * This parses the progammer specifier @s, which can either be a single
  * filename, or a comma-separated series of <id>:<filename> entries.
  *
- * In the first case @images[0] is assigned the provided filename and @single is
- * set to true. In the second case, each comma-separated entry will be split on
- * ':' and the given <filename> will be assigned to the @image entry indicated
- * by the given <id>.
+ * In the first case an attempt will be made to decode the Sahara archive and
+ * each programmer part will be loaded into their requestd @images entry. If
+ * the file isn't an archive @images[SAHARA_ID_EHOSTDL_IMG] is assigned. In the
+ * second case, each comma-separated entry will be split on ':' and the given
+ * <filename> will be assigned to the @image entry indicated by the given <id>.
  *
  * Memory is not allocated for the various strings, instead @s will be modified
  * by the tokenizer and pointers to the individual parts will be stored in the
@@ -360,57 +372,50 @@ err_free_doc:
  *
  * Returns: 0 on success, -1 otherwise.
  */
-static int decode_programmer(char *s, struct sahara_image *images, bool *single)
+static int decode_programmer(char *s, struct sahara_image *images)
 {
+	struct sahara_image archive;
 	char *filename;
 	char *save1;
-	char *save2;
 	char *pair;
-	char *id_str;
+	char *tail;
 	long id;
 	int ret;
 
-	if (!strchr(s, ':')) {
-		ret = load_sahara_image(s, &images[0]);
-		if (ret < 0)
-			return -1;
+	strtoul(s, &tail, 0);
+	if (tail != s && tail[0] == ':') {
+		for (pair = strtok_r(s, ",", &save1); pair; pair = strtok_r(NULL, ",", &save1)) {
+			id = strtoul(pair, &tail, 0);
+			if (tail == pair) {
+				ux_err("invalid programmer specifier\n");
+				return -1;
+			}
 
-		ret = decode_programmer_archive(images);
-		if (ret < 0)
-			return -1;
+			if (id == 0 || id >= MAPPING_SZ) {
+				ux_err("invalid image id \"%s\"\n", pair);
+				return -1;
+			}
 
-		if (!ret) {
-			ret = decode_sahara_config(images);
+			filename = &tail[1];
+			ret = load_sahara_image(filename, &images[id]);
 			if (ret < 0)
 				return -1;
 		}
-
-		*single = (ret == 0);
-
-		return 0;
-	}
-
-	for (pair = strtok_r(s, ",", &save1); pair; pair = strtok_r(NULL, ",", &save1)) {
-		id_str = strtok_r(pair, ":", &save2);
-		filename = strtok_r(NULL, ":", &save2);
-
-		if (!id_str || !filename) {
-			ux_err("failed to parse programmer specifier\n");
-			return -1;
-		}
-
-		id = strtoul(id_str, NULL, 0);
-		if (id == 0 || id >= MAPPING_SZ) {
-			ux_err("invalid image id \"%s\"\n", id_str);
-			return -1;
-		}
-
-		ret = load_sahara_image(filename, &images[id]);
+	} else {
+		ret = load_sahara_image(s, &archive);
 		if (ret < 0)
 			return -1;
-	}
 
-	*single = false;
+		ret = decode_programmer_archive(&archive, images);
+		if (ret < 0 || ret == 1)
+			return ret;
+
+		ret = decode_sahara_config(&archive, images);
+		if (ret < 0 || ret == 1)
+			return ret;
+
+		images[SAHARA_ID_EHOSTDL_IMG] = archive;
+	}
 
 	return 0;
 }
@@ -421,6 +426,8 @@ static void print_usage(FILE *out)
 
 	fprintf(out, "Usage: %s [options] <prog.mbn> (<program-xml> | <patch-xml> | <read-xml>)...\n", __progname);
 	fprintf(out, "       %s [options] <prog.mbn> ((read | write) <address> <binary>)...\n", __progname);
+	fprintf(out, "       %s list\n", __progname);
+	fprintf(out, "       %s ramdump [--debug] [-o <ramdump-path>] [<segment-filter>,...]\n", __progname);
 	fprintf(out, " -d, --debug\t\t\tPrint detailed debug info\n");
 	fprintf(out, " -v, --version\t\t\tPrint the current version and exit\n");
 	fprintf(out, " -n, --dry-run\t\t\tDry run execution, no device reading or flashing\n");
@@ -434,21 +441,122 @@ static void print_usage(FILE *out)
 	fprintf(out, " -T, --slot=T\t\t\tSet slot number T for multiple storage devices\n");
 	fprintf(out, " -D, --vip-table-path=T\t\tUse digest tables in the T folder for VIP\n");
 	fprintf(out, " -h, --help\t\t\tPrint this usage info\n");
-	fprintf(out, " <program-xml>\txml file containing <program> or <erase> directives\n");
-	fprintf(out, " <patch-xml>\txml file containing <patch> directives\n");
-	fprintf(out, " <read-xml>\txml file containing <read> directives\n");
-	fprintf(out, " <address>\tdisk address specifier, can be one of <P>, <P/S>, <P/S+L>, <name>, or\n");
-	fprintf(out, "          \t<P/name>, to specify a physical partition number P, a starting sector\n");
-	fprintf(out, "          \tnumber S, the number of sectors to follow L, or partition by \"name\"\n");
+	fprintf(out, " <program-xml>\t\txml file containing <program> or <erase> directives\n");
+	fprintf(out, " <patch-xml>\t\txml file containing <patch> directives\n");
+	fprintf(out, " <read-xml>\t\txml file containing <read> directives\n");
+	fprintf(out, " <address>\t\tdisk address specifier, can be one of <P>, <P/S>, <P/S+L>, <name>, or\n");
+	fprintf(out, "          \t\t<P/name>, to specify a physical partition number P, a starting sector\n");
+	fprintf(out, "          \t\tnumber S, the number of sectors to follow L, or partition by \"name\"\n");
+	fprintf(out, " <ramdump-path>\t\tpath where ramdump should stored\n");
+	fprintf(out, " <segment-filter>\toptional glob-pattern to select which segments to ramdump\n");
 	fprintf(out, "\n");
 	fprintf(out, "Example: %s prog_firehose_ddr.elf rawprogram*.xml patch*.xml\n", __progname);
 }
 
-int main(int argc, char **argv)
+static int qdl_list(FILE *out)
+{
+	struct qdl_device_desc *devices;
+	unsigned int count;
+	unsigned int i;
+
+	devices = usb_list(&count);
+	if (!devices)
+		return 1;
+
+	if (count == 0) {
+		fprintf(out, "No devices found\n");
+	} else {
+		for (i = 0; i < count; i++)
+			fprintf(out, "%04x:%04x\t%s\n",
+				devices[i].vid, devices[i].pid, devices[i].serial);
+	}
+
+	free(devices);
+
+	return 0;
+}
+
+static int qdl_ramdump(int argc, char **argv)
+{
+	struct qdl_device *qdl;
+	char *ramdump_path = ".";
+	char *filter = NULL;
+	char *serial = NULL;
+	int ret = 0;
+	int opt;
+
+	static struct option options[] = {
+		{"debug", no_argument, 0, 'd'},
+		{"version", no_argument, 0, 'v'},
+		{"output", required_argument, 0, 'o'},
+		{"serial", required_argument, 0, 'S'},
+		{"help", no_argument, 0, 'h'},
+		{0, 0, 0, 0}
+	};
+
+	while ((opt = getopt_long(argc, argv, "dvo:S:h", options, NULL)) != -1) {
+		switch (opt) {
+		case 'd':
+			qdl_debug = true;
+			break;
+		case 'v':
+			print_version();
+			return 0;
+		case 'o':
+			ramdump_path = optarg;
+			break;
+		case 'S':
+			serial = optarg;
+			break;
+		case 'h':
+			print_usage(stdout);
+			return 0;
+		default:
+			print_usage(stderr);
+			return 1;
+		}
+	}
+
+	if (optind < argc)
+		filter = argv[optind++];
+
+	if (optind != argc) {
+		print_usage(stderr);
+		return 1;
+	}
+
+	ux_init();
+
+	qdl = qdl_init(QDL_DEVICE_USB);
+	if (!qdl)
+		return 1;
+
+	if (qdl_debug)
+		print_version();
+
+	ret = qdl_open(qdl, serial);
+	if (ret) {
+		ret = 1;
+		goto out_cleanup;
+	}
+
+	ret = sahara_run(qdl, NULL, ramdump_path, filter);
+	if (ret < 0) {
+		ret = 1;
+		goto out_cleanup;
+	}
+
+out_cleanup:
+	qdl_close(qdl);
+	qdl_deinit(qdl);
+
+	return ret;
+}
+
+static int qdl_flash(int argc, char **argv)
 {
 	enum qdl_storage_type storage_type = QDL_STORAGE_UFS;
 	struct sahara_image sahara_images[MAPPING_SZ] = {};
-	bool single_image = true;
 	char *incdir = NULL;
 	char *serial = NULL;
 	const char *vip_generate_dir = NULL;
@@ -570,7 +678,7 @@ int main(int argc, char **argv)
 	if (qdl_debug)
 		print_version();
 
-	ret = decode_programmer(argv[optind++], sahara_images, &single_image);
+	ret = decode_programmer(argv[optind++], sahara_images);
 	if (ret < 0)
 		exit(1);
 
@@ -635,7 +743,7 @@ int main(int argc, char **argv)
 
 	qdl->storage_type = storage_type;
 
-	ret = sahara_run(qdl, sahara_images, single_image, NULL, NULL);
+	ret = sahara_run(qdl, sahara_images, NULL, NULL);
 	if (ret < 0)
 		goto out_cleanup;
 
@@ -651,6 +759,9 @@ out_cleanup:
 		vip_gen_finalize(qdl);
 
 	qdl_close(qdl);
+
+	sahara_images_free(sahara_images, MAPPING_SZ);
+
 	free_programs();
 	free_patches();
 
@@ -661,3 +772,13 @@ out_cleanup:
 
 	return !!ret;
 }
+
+int main(int argc, char **argv)
+{
+	if (argc == 2 && !strcmp(argv[1], "list"))
+		return qdl_list(stdout);
+	if (argc >= 2 && !strcmp(argv[1], "ramdump"))
+		return qdl_ramdump(argc - 1, argv + 1);
+
+	return qdl_flash(argc, argv);
+}
