@@ -76,6 +76,8 @@ int main(int argc, char **argv)
 		return 1;
 	}
 
+	ux_init();
+
 	if (qdl_debug)
 		print_version();
 
@@ -85,7 +87,7 @@ int main(int argc, char **argv)
 		goto out_cleanup;
 	}
 
-	ret = sahara_run(qdl, NULL, true, ramdump_path, filter);
+	ret = sahara_run(qdl, NULL, ramdump_path, filter);
 	if (ret < 0) {
 		ret = 1;
 		goto out_cleanup;
